#include "VirtualRobot.h"

/**
 ************************************************************************
 * Abre un mapa predefinido por el usuario.  El realidad esta funcin abre un mapa  
en blanco, lee el fichero de mundo especificado, y luego pinta el mundo.
 * @param WindowHandle Handle de la ventana que contendra el mapa
 * @param FICHERO Path completo con el nombre del fichero del mapa
 * @param DIRECTORIO_OBJETOS Directorio donde se encuentran los ficheros de los objetos 
 del mapa
 * @param alto Altura de la ventana donde ira el mapa expresada en pixeles
 * @param ancho Ancho de la ventana donde ira el mapa expresada en pixeles
 * @param Escala Escalado que tendra la imagen. En milimetros/pixel
 * @param PosX Devuelve la coordenada X que ocupar el robot en el mapa. Esta
 coordenada se lee del fichero de Mundo y debe estar expresada en milimetros
 * @param PosY Devuelve la coordenada Y que ocupar el robot en el mapa.Esta
 coordenada se lee del fichero de Mundo y debe estar expresada en milimetros
 * @param Heading Devuelve la orientacion que tendra el robot
 * @param Sys Sistema en que se alocatarn los Buffers
 * @param Disp Devuelve la direccion del Display alocatado donde se
representar el mapa
 * @param mapa Devuelve la direccion del Buffer alocatado donde se
guardar el mapa
 * @see NuevoMapaMIL()
 * @see SalvarMapaMIL()
 * @see RecalcularMapaMIL()
 * @see CerrarMapaMIL()
 * @see LeerMundo()
 **************************************************************************/
DLL_EXPORT void STD AbrirMapaMIL (HWND WindowHandle,char *FICHERO,char *DIRECTORIO_OBJETOS, 
								  long *alto,long *ancho,double escala,double *PosX,double *PosY,
								  double *Heading,MIL_ID sys,MIL_ID *disp,MIL_ID *mapa)
{
	int n;
	Pt2D tmp;
	double tmp2;

// Inicializamos todos los punteros a objetos como NULL
	for (n=0;n<30;n++) Mundo[n]=NULL;

	MatrizObjetos=IniMatrix(3);
	NuevoMapaMIL(WindowHandle,*alto,*ancho,escala,sys,disp,mapa);
	LeerMundo(Mundo,FICHERO,DIRECTORIO_OBJETOS,&tmp,&tmp2);
	*PosX=tmp.x;
	*PosY=tmp.y;
	*Heading=tmp2;
	PintarMundo(Mundo,ESTACIONARIO);
	MbufCopy(WorkingBuffer,RefreshBuffer);
	}

/**
 *************************************************************************
 * Abre un mapa en blanco. En esta funcin es donde se alocatan
todos los buffers necesarios para el funcionamiento del programa.Tambien
se calcula la matriz de cambio de base en funcin de los parmetros del mapa
 * @param WindowHandle Handle de la ventana que contendra el mapa
 * @param alto Altura de la ventana donde ira el mapa
 * @param ancho Ancho de la ventana donde ira el mapa
 * @param Escala Escalado que tendra la imagen. En milimetros/pixel
 * @param Sys Sistema en que se alocatarn los Buffers
 * @param Disp Devuelve la direccion del Display alocatado donde se
representar el mapa
 * @param mapa Devuelve la direccion del Buffer alocatado donde se
 * @see AbrirMapaMIL()
 * @see SalvarMapaMIL()
 * @see RecalcularMapaMIL()
 * @see CerrarMapaMIL()
 *************************************************************************/
DLL_EXPORT void STD NuevoMapaMIL (HWND WindowHandle,long alto, long ancho ,
								  double escala,MIL_ID sys,MIL_ID *disp,
								  MIL_ID *mapa)
{
//	Alocatamos los displays y buffers necesarios
	MdispAlloc (sys,M_DEFAULT,"M_DEFAULT",M_DEFAULT,&*disp);
//	MdispAlloc (sys,M_DEFAULT,"M_DEFAULT",M_DEFAULT,&WorkingDisplay);	//Muestra el Buffer de trabajo
//	MdispAlloc (sys,M_DEFAULT,"M_DEFAULT",M_DEFAULT,&RefreshDisplay);	//Muestra el Buffer de refresco
//	MdispAlloc (sys,M_DEFAULT,"M_DEFAULT",M_DEFAULT,&SonaresDisplay);	//Muestra el Buffer del sonar
	MbufAllocColor (sys,3,ancho,alto,8,M_IMAGE+M_DISP+M_PROC,&*mapa);
	MbufAllocColor (sys,3,ancho,alto,8,M_IMAGE+M_DISP+M_PROC,&WorkingBuffer);
	MbufAllocColor (sys,3,ancho,alto,8,M_IMAGE+M_DISP+M_PROC,&RefreshBuffer);
//	MbufAllocColor (sys,3,ancho,alto,8,M_IMAGE+M_DISP+M_PROC,&SonaresBuffer);
	MbufClear (*mapa,255);
	MdispSelectWindow (*disp,*mapa,WindowHandle);
//	MdispSelect (WorkingDisplay,WorkingBuffer);
//	MdispSelect (RefreshDisplay,RefreshBuffer);
//	MdispSelect (SonaresDisplay,SonaresBuffer);
	MdispControl (*disp,M_WINDOW_ZOOM,M_DISABLE);
	MdispControl (*disp,M_WINDOW_TITLE_BAR,M_DISABLE);
	MilUnderlayBuffer=*mapa;
	MgraColor (M_DEFAULT, 0);


	Buffer_Width=ancho;
	Buffer_Height=alto;
	MbufClear(RefreshBuffer,255);
	ESCALA_PixPorMM=1/escala;

//	Definimos una Zona de Alta densidad por defecto en caso de que
//	el usuario habilite el uso de esta antes de definir una el mismo
	ZonaAltaDensidad.EsqSupIzq.x =0;
	ZonaAltaDensidad.EsqSupIzq.y =Buffer_Height*escala;
	ZonaAltaDensidad.EsqInfDrch.x =(Buffer_Width*escala)/2; 
	ZonaAltaDensidad.EsqInfDrch.y =(Buffer_Height*escala)/2; 

//	Calculamos la matriz de cambio de base y su inversa
	AlturaImagen=(double)alto*escala;
	Base=IniciarMatrizCambioBase (&Giro,&Traslacion,&Simetria_X,AlturaImagen,0,0,0,0,0);
	InversaBase=Inversa3X3(Base);
}

/**
 **************************************************************************************
 * Esta funcin salva en disco una imagen del buffer de refresco en formato BMP
 * @param FileName Path completo con el nombre del fichero en el que se salva el buffer
de refresco
 * @see AbrirMapaMIL()
 * @see NuevoMapaMIL()
 * @see RecalcularMapaMIL()
 * @see CerrarMapaMIL()
 **************************************************************************************/
DLL_EXPORT void STD SalvarMapaMIL (char *FileName)
{
	MbufExport (FileName,M_BMP,RefreshBuffer);
}

/**
 ***********************************************************************************
 * Recalcula el mapa. Recalcula la representacin del mapa en funcin de los parmetros
 de escala, desplazamiento, giro del mapa, etc...
 * @param escala Escala de representacin del mapa en milmetro por pxel
 * @param DesplazamientoH Desplazamiento en el eje x de la representacin del mapa.  
 El desplazamiento se realiza en coordenadas de mundo.
 * @param DesplazamientoV Desplazamiento en el eje y de la representacin del mapa.  
 El desplazamiento se realiza en coordenadas de mundo.
 * @param AnguloGiro ngulo en que se representar el mapa
 * @param EjeGiroX Coordenada X del punto de mundo alrededor del cual 
 se realizar el giro del mapa.
 * @param EjeGiroY Coordenada Y del punto de mundo alrededor del cual 
 se realizar el giro del mapa.
 * @param modo Manera en que se representan los objetos en el plano.  Si modo=0=MOVIMIENTO
 entonces el contorno de la habitacin y los objetos no se rellenan; si modo=1=ESTACIONARIO
 los objetos y el contorno de la habitacin se rellenan de un color.
 * @see AbrirMapaMIL()
 * @see NuevoMapaMIL()
 * @see SalvarMapaMIL()
 * @see CerrarMapaMIL()
 *********************************************************************************/
DLL_EXPORT void STD RecalcularMapaMIL (double escala,double DesplazamientoH,
									   double DesplazamientoV,double AnguloGiro,
									   long EjeGiroX,long EjeGiroY,int modo)
{
	double alto;

//	Recalculamos la Matriz de cambio de base y su inversa
	alto=AlturaImagen*ESCALA_PixPorMM;
	ESCALA_PixPorMM=1/escala;
	AlturaImagen=(double)alto*escala;
//	modo=ESTACIONARIO;	esta linea esta aqui para pruebas
	Base=IniciarMatrizCambioBase (&Giro,&Traslacion,&Simetria_X,AlturaImagen,
									DesplazamientoH,DesplazamientoV,AnguloGiro,
									EjeGiroX,EjeGiroY);
	InversaBase=Inversa3X3(Base);

//	Repintamos la toda escena
	MbufClear (WorkingBuffer,255);	//Pintamos el WorkingBuffer de blanco
									//para que en caso de no pintar ningun mundo
									//no se produzcan errores
	PintarMundo (Mundo,	modo);
	MbufCopy(WorkingBuffer,RefreshBuffer);
	PintaRobot (MatrizRobot);
	PintaCamaras (MatrizCamaras);
	PintaZona(ZonaAltaDensidad);
	ObtenerLecturaSonares();
	MbufCopy(WorkingBuffer,MilUnderlayBuffer);
}

/**
 ***************************************************************************
 * Desalocata los buffers utilizados para representar un mapa, asi como
 la estructura de datos del mundo
 * @param Disp Display que estaba alocatado
 * @see AbrirMapaMIL()
 * @see NuevoMapaMIL()
 * @see SalvarMapaMIL()
 * @see RecalcularMapaMIL()
 ****************************************************************************/
DLL_EXPORT void STD CerrarMapaMIL (MIL_ID disp)
{
	MbufFree(MilUnderlayBuffer);
	MbufFree(WorkingBuffer);
	MbufFree(RefreshBuffer);
//	MbufFree(SonaresBuffer);
	BorrarMundo(Mundo);
	MdispFree(disp);
}
/**
 ***************************************************************************
 * Habilita el uso de la ventana de Resolucin y Repinta el mundo junto con
 la ventana
 * @see AjustarZonaAltaResolucion()
 * @see NoUsarZonaAltaResolucion()
 **************************************************************************/
DLL_EXPORT void STD UsarZonaAltaResolucion (void)	
{
	ZonaAltaDensidad.UsarZona=true;
	MbufCopy(RefreshBuffer,WorkingBuffer);
	PintaRobot(MatrizRobot);
	PintaCamaras(MatrizCamaras);
	PintaZona(ZonaAltaDensidad);
	MbufCopy(WorkingBuffer,MilUnderlayBuffer);
}

/**
 ***************************************************************************
 * Ajusta el tamao de la ventana de resolucin
 * @param XsupIzq Coordenada X de la esquina superior izquierda de 
 la ventana de resolucin.  Coordenada expresada en coordenadas de mundo
 * @param YsupIzq Coordenada Y de la esquina superior izquierda de 
 la ventana de resolucin.  Coordenada expresada en coordenadas de mundo
 * @param XInfDrch Coordenada X de la esquina inferior derecha de 
 la ventana de resolucin.  Coordenada expresada en coordenadas de mundo
 * @param YInfDrch Coordenada Y de la esquina inferior derecha de 
 la ventana de resolucin.  Coordenada expresada en coordenadas de mundo
 * @see NoUsarZonaAltaResolucion()
 * @see UsarZonaAltaResolucion()
 **************************************************************************/
DLL_EXPORT void STD AjustarZonaAltaResolucion (long XSupIzq,long YSupIzq, 
										   long XInfDrch,long YInfDrch)
{
	ZonaAltaDensidad.EsqSupIzq.x=XSupIzq;		
	ZonaAltaDensidad.EsqSupIzq.y=YSupIzq;		
	ZonaAltaDensidad.EsqInfDrch.x=XInfDrch;
	ZonaAltaDensidad.EsqInfDrch.y=YInfDrch;
	ZonaAltaDensidad.UsarZona =true;
	MbufCopy(RefreshBuffer,WorkingBuffer);
	PintaRobot (MatrizRobot);
	PintaCamaras (MatrizCamaras);
	PintaZona(ZonaAltaDensidad);
	MbufCopy(WorkingBuffer,MilUnderlayBuffer);	//Copiamos al buffer que se muestra
}

/**
 ***************************************************************************
 * Deshabilita el uso de la ventana de resolucin y repinta el mundo pero 
 sin la ventana
 * @see UsarZonaAltaResolucion()
 * @see AjustarZonaAltaResolucion()
 **************************************************************************/
DLL_EXPORT void STD NoUsarZonaAltaResolucion (void)	
{
	ZonaAltaDensidad.UsarZona=false;
	MbufCopy(RefreshBuffer,WorkingBuffer);
	PintaRobot(MatrizRobot);
	PintaCamaras(MatrizCamaras);
	MbufCopy(WorkingBuffer,MilUnderlayBuffer);
}

/**
 ***************************************************************************
 * Define la forma del robot virtual y lo posiciona por primera 
 en el mapa 
 * @param x Coordenada X de la posicin que ocupar el robot en el mapa
 * @param y Coordenada Y de la posicin que ocupar el robot en el mapa
 * @param Angulo Orientacin del robot en el mapa
 * @param AnguloCamaras Orientacin de la camaras respecto al robot
 * @see ActualizarVista()
 * @see IniciarMatrices()
 **************************************************************************/
DLL_EXPORT void STD PosicionarRobotVirtual (double x, double y, double Angulo,
											double AnguloCamaras, long modo)
{
//	PosicionActual.x=x;
//	PosicionActual.y=y;

//	Definimos fsicamente el robot
	Robot.P1.x=99.8;				Robot.P1.y=190;
	Robot.P2.x=190;					Robot.P2.y=99.8;
	Robot.P3.x=190;					Robot.P3.y=-99.8;
	Robot.P4.x=99.8;				Robot.P4.y=-190;
	Robot.P5.x=-99.8;				Robot.P5.y=-190;
	Robot.P6.x=-190;				Robot.P6.y=-99.8;
	Robot.P7.x=-190;				Robot.P7.y=99.8;
	Robot.P8.x=-99.8;				Robot.P8.y=190;
	Robot.centro.x=0;				Robot.centro.y=0;
	Robot.BaseFlecha.x=0;			Robot.BaseFlecha.y=190;
	Robot.PuntaFlechaC.x=0;			
	Robot.PuntaFlechaC.y=(30*(1/ESCALA_PixPorMM))+190;

	Robot.PuntaFlechaL.x=-5*(1/ESCALA_PixPorMM);	
	Robot.PuntaFlechaL.y=(25*(1/ESCALA_PixPorMM))+190;
	Robot.PuntaFlechaR.x=5*(1/ESCALA_PixPorMM);	
	Robot.PuntaFlechaR.y=(25*(1/ESCALA_PixPorMM))+190;

//	Definimos fsicamente las camaras
    camaras.uno.x=0;				camaras.uno.y = 49.9;
    camaras.dos.x=29.9;				camaras.dos.y = -29.9;
    camaras.tres.x=0;				camaras.tres.y = 0;
    camaras.cuatro.x=-29.9;			camaras.cuatro.y = -29.9;
	camaras.Cono1L.x=-20*(1/ESCALA_PixPorMM);	
	camaras.Cono1L.y=90*(1/ESCALA_PixPorMM);
	camaras.Cono1R.x=20*(1/ESCALA_PixPorMM);		
	camaras.Cono1R.y=90*(1/ESCALA_PixPorMM);

	IniciarMatrices (&Giro,&Traslacion,&MatrizRobot,&MatrizCamaras,
		&PosicionActual,Angulo, AnguloCamaras);

	ActualizarVista(x,y,Angulo,AnguloCamaras,modo);
}

/**
 ***************************************************************************
 * Actualiza la vista del mundo. Pinta el mapa, el robot, las camaras, la ventana
 de resolucin y las lecturas de los sonares
 * @param x Coordenada X de la posicin que ocupar el robot en el mapa
 * @param y Coordenada Y de la posicin que ocupar el robot en el mapa
 * @param Angulo Orientacin del robot en el mapa
 * @param AnguloCamaras Orientacin de la camaras respecto al robot
 * @see PintaRobot()
 * @see PintaCamaras()
 * @see PintaZona()
 * @see ObtenerLecturaSonares()
 **************************************************************************/
DLL_EXPORT void STD ActualizarVista (double x, double y, double Angulo
									   ,double AnguloCamaras, long modo)
{
	PosicionActual.x=x;
	PosicionActual.y=y;
	switch (modo)
	{
		case 0:
			//	Centrado en el Mundo
			IniTraslacion(Traslacion,PosicionActual.x,PosicionActual.y);
			IniGiro(Giro,Angulo);
			mulmat (Giro,Traslacion,MatrizRobot);
			IniMatrizCamaras(MatrizCamaras,Giro,Traslacion,AnguloCamaras);
			PintaRobot (MatrizRobot);
			PintaCamaras (MatrizCamaras);
			PintaZona(ZonaAltaDensidad);
			ObtenerLecturaSonares();
			MbufCopy(WorkingBuffer,MilUnderlayBuffer);	
			break;
		case 1:
			//	Centrado en el Robot
			double CentradoX, CentradoY;

			CentradoX=PosicionActual.x-(Buffer_Width/(2*ESCALA_PixPorMM));
			CentradoY=PosicionActual.y-(Buffer_Height/(2*ESCALA_PixPorMM));

			Base=IniciarMatrizCambioBase (&Giro,&Traslacion,&Simetria_X,(Buffer_Height/ESCALA_PixPorMM),
											-CentradoX,-CentradoY,Angulo,
											PosicionActual.x,PosicionActual.y);
			InversaBase=Inversa3X3(Base);


			MbufClear (WorkingBuffer,255);	//Pintamos el WorkingBuffer de blanco
											//para que en caso de no pintar ningun mundo
											//no se produzcan errores
			PintarMundo (Mundo,	MOVIMIENTO);
			MbufCopy(WorkingBuffer,RefreshBuffer);

			IniTraslacion(Traslacion,PosicionActual.x,PosicionActual.y);
			IniGiro(Giro,Angulo);
			mulmat (Giro,Traslacion,MatrizRobot);
			IniMatrizCamaras(MatrizCamaras,Giro,Traslacion,AnguloCamaras);
			PintaRobot (MatrizRobot);
			PintaCamaras (MatrizCamaras);
			PintaZona(ZonaAltaDensidad);
			ObtenerLecturaSonares();
			MbufCopy(WorkingBuffer,MilUnderlayBuffer);
			break;
	}	
}

/**
 ***************************************************************************
 * Actualiza la lista de las lecturas de los sonares y los pinta
 * @see VaciarBufferLecturasSonares()
 * @see s_actualizar()
 **************************************************************************/
DLL_EXPORT void STD ObtenerLecturaSonares (void)
{
	Pt2D	PuntoPantalla;
	int		j;

//	Actualizamos la lista de lecturas con otras nuevas
	s_actualitzar();
	MgraColor (M_DEFAULT,M_RGB888(17,140,255));
	for (j=0;j<LISTA_MAX;j++)
	{
		PuntoPantalla=MultVect(Base,ListaFIFO[j]);
		MgraArc (M_DEFAULT,WorkingBuffer,(long)PuntoPantalla.x,(long)PuntoPantalla.y,5,5,0,360);
	}
	MgraColor (M_DEFAULT,0);
}

/**
 ***************************************************************************
 * Vacia la listas de lecturas tanto las internas de Saphira,como las de la
propia aplicacin
 * @see ObtenerLecturaSonares()
 **************************************************************************/
DLL_EXPORT void STD VaciarBufferLecturasSonares (void)
{
	int i;
	Pt2D	Lectura;

	Lectura.x=0;	Lectura.y=0;
	for (i=0;i<LISTA_MAX;i++)
	{
		ListaFIFO[i]=Lectura;
	}
	sfSetFrontBuffer (0);	//vacia el buffer frontal y trasero de Saphira
	sfSetSideBuffer (0);	//vacia los buffers laterales de Saphira
}
/*
DLL_EXPORT void STD Tratar_Lecturas_Sonares (void)
{
	int		i;
	Pt2D	Punto_Pantalla;

//	MbufClear(MilOverlayBuffer, TransparentColor);
	MgraColor (M_DEFAULT,M_RGB888(17,140,255));
	for (i=0;i<LISTA_MAX;i++)
	{
		if (ListaFIFO[i].x!=0 && ListaFIFO[i].y!=0)
		{
			Punto_Pantalla=MultVect(Base,ListaFIFO[i]);
			MgraArc (M_DEFAULT,WorkingBuffer,(long)Punto_Pantalla.x,(long)Punto_Pantalla.y,5,5,0,360);
		}
	}		
	MgraColor (M_DEFAULT,0);
}
*/

/**
 ***************************************************************************
 * Transforma un punto de coordenadas de Mundo a coordenadas de dispositivo
 teniendo en cuenta la matriz de cambio de base que ya hay calculada
 * @param PosXMundo Coordenada X del punto de mundo
 * @param PosYMundo Coordenada Y del punto de mundo
 * @param PosXDisp Coordenada X del punto de dispositivo
 * @param PosYDisp Coordenada Y del punto de dispositivo
 * @see TransfDispositivoMundo();
 **************************************************************************/
DLL_EXPORT void STD TransfMundoDispositivo (double PosXMundo, double PosYMundo,
											double *PosXDisp, double *PosYDisp)
{
	Pt2D	PuntoMundo,PuntoDisp;

	PuntoMundo.x=PosXMundo;
	PuntoMundo.y=PosYMundo;
	PuntoDisp=MultVect(Base,PuntoMundo);
	*PosXDisp=PuntoDisp.x;
	*PosYDisp=PuntoDisp.y;
}

/**
 ***************************************************************************
 * Transforma un punto de coordenadas de Mundo a coordenadas de dispositivo
 teniendo en cuenta la matriz de cambio de base que ya hay calculada
 * @param PosXMundo Coordenada X del punto de mundo
 * @param PosYMundo Coordenada Y del punto de mundo
 * @param PosXDisp Coordenada X del punto de dispositivo
 * @param PosYDisp 
 * @see TransfDispositivoMundo();
 **************************************************************************/
DLL_EXPORT void STD TransfDispositivoMundo (double PosXDisp, double PosYDisp,
											double *PosXMundo, double *PosYMundo)
{
	Pt2D	PuntoMundo,PuntoDisp;

	PuntoDisp.x=PosXDisp;
	PuntoDisp.y=PosYDisp;
	PuntoMundo=MultVect(InversaBase,PuntoDisp);
	*PosXMundo=PuntoMundo.x;
	*PosYMundo=PuntoMundo.y;
}


/**
 ***************************************************************************
 * Traza una recta entre dos puntos dados en coordenadas de mundo. Esta recta 
 se trazar tanto en el buffer de visible como en el de refresco, de manera
 que hasta que no se recalcule el mapa aparecer en la imagen.
 * @param x1 Coordenada X del primer punto
 * @param y1 Coordenada Y del primer punto
 * @param x2 Coordenada X del segundo punto
 * @param y2 Coordenada Y del segundo punto
 * @see PintarPuntoTrayectoria() 
 **************************************************************************/
DLL_EXPORT void STD SplineTracer (double x1, double y1, double x2, double y2)
{
	Pt2D punto,P1,P2;

	punto.x=x1;
	punto.y=y1;
	P1=MultVect(Base,punto);
	punto.x=x2;
	punto.y=y2;
	P2=MultVect(Base,punto);

	MgraColor (M_DEFAULT,0); 
	MgraLine(M_DEFAULT,RefreshBuffer,P1.x,P1.y,P2.x,P2.y);
	MgraLine(M_DEFAULT,MilUnderlayBuffer,P1.x,P1.y,P2.x,P2.y);
}

/**
 ***************************************************************************
 * Pinta un punto rojo en un punto dado en coordenadas de mundo. Este punto 
 se repesentar tanto en el buffer de visible como en el de refresco, de manera
 que hasta que no se recalcule el mapa aparecer en la imagen.
 * @param x1 Coordenada X del punto
 * @param y1 Coordenada Y del punto
 * @see SplineTracer()
 **************************************************************************/
DLL_EXPORT void STD PintarPuntoTrayectoria (double x1, double y1)
{
	Pt2D punto, resultado;

	punto.x=x1;
	punto.y=y1;
	resultado=MultVect(Base,punto);

	MgraColor (M_DEFAULT,M_RGB888(200,0,0));
	MgraDot(M_DEFAULT,RefreshBuffer,resultado.x,resultado.y);
	MgraDot(M_DEFAULT,MilUnderlayBuffer,resultado.x,resultado.y);
}

/**
 ***************************************************************************
 * Funcin para debugar. Innecesaria para el funcionamiento del programa
 **************************************************************************/
DLL_EXPORT void STD GrabarBuffer ()
{
//	FILE	*fichero;
	int		i;
	char	nombre[15];
	char	tmp[3];

	strcpy (nombre,"c:\\mio0");
	i=0;
	_itoa (i,tmp,10);
	strcat (nombre,tmp);
	strcat (nombre,".tif");
	while (fopen(nombre,"r")!=NULL) 
	{
		i++;
		_itoa (i,tmp,10);
		if (i<10){	strcpy (nombre,"c:\\mio0"); }
		else if (i>=10){strcpy (nombre,"c:\\mio");}
		strcat (nombre,tmp);
		strcat (nombre,".tif");
	}
	MbufExport(nombre,M_TIFF,MilUnderlayBuffer);
}

/***************************************************
	FUNCIONES DE BAJO NIVEL
****************************************************/

/**
 ***************************************************************************
 * Reserva espacio en memoria para todas las matrices necesarias en el calculo
de transformaciones 2D del programa.Tambien calcula la matriz de cambio de base 
necesaria para transformar un punto de mundo en un punto de dispositivo
 * @param giro Devuelve el puntero a una matriz de Giro
 * @param trasl Devuelve el puntero a una matriz de traslacin
 * @param simetria_x Devuelve el puntero a una matriz de simetra respecto al eje X
 * @param AlturaImagen Altura de la ventana en que se va a representar el mapa medida en pxeles
 * @param DesplazamientoH Desplazamiento horizontal que queremos que tenga 
 la representacin del mapa
 * @param DesplazamientoV Desplazamiento vertical que queremos que tenga 
 la representacin del mapa
 * @param AnguloGiro ngulo con el que queremos que se represente el mapa
 * @param EjeGiroX Coordenada X del punto respecto al que queremos que se 
 realice el giro del mapa
 * @param EjeGiroY Coordenada Y del punto respecto al que queremos que se 
 realice el giro del mapa
 * @return La matriz de cambio de base
 **************************************************************************/
matriz IniciarMatrizCambioBase (matriz *giro,matriz *trasl,matriz *simetria_x
									 ,double AlturaImagen, double DesplazamientoH,
									 double DesplazamientoV,double AnguloGiro,
									 long EjeGiroX,long EjeGiroY)
{
	matriz a,b,c,d;
	matriz temp,MatrizEscalado,Resultado;

	temp=IniMatrix (3);		Resultado=IniMatrix(3);
	a=IniMatrix (3);		b=IniMatrix (3);
	c=IniMatrix (3);		d=IniMatrix (3);
	MatrizEscalado=IniMatrix(3);

//	Calculos necesarios para la obtencion de la matriz de cambio de base
	IniTraslacion (b,DesplazamientoH,AlturaImagen-DesplazamientoV);
	IniGiroRespectoPunto (a,AnguloGiro,-EjeGiroX,-EjeGiroY);
	IniSimetriaX (c);
	IniEscalado (ESCALA_PixPorMM,ESCALA_PixPorMM,MatrizEscalado);
//	IniEscaladoRespectoPunto (ESCALA_PixPorMM,ESCALA_PixPorMM,
//							EjeGiroX,EjeGiroY,MatrizEscalado);
	mulmat (c,a,temp);
	mulmat (temp,b,d);
	mulmat (d,MatrizEscalado,Resultado);
	*giro=a;			*trasl=b;
	*simetria_x=c;
	
	FreeMatrix(temp,3);
	FreeMatrix(d,3);

	return(Resultado);
}

/**
 ***************************************************************************
 * Reserva espacio en memoria para las matrices que almacenan las trasformaciones
2D necesarias para representar el robot y la informacin de las camaras por 
primera vez.
 * @param giro Devuelve el apuntador a la matriz que gira el robot
 * @param trasl Devuelve el apuntador a la matriz que traslada el robot
 * @param MatrizRobot Devuelve el apuntador a la matriz que contiene las
transformaciones 2D necesarias para posicionar el robot en el mundo
 * @param matrizcamaras Devuelve el apuntador a la matriz que contiene las
transformaciones 2D necesarias para posicionar la informacin de las 
cmaras en el mundo
 * @param PosicionActual Punto 2D que nos indica cual es la posicin actual del
robot en coordenadas de mundo
 * @param AnguloActual ngulo actual del robot
 * @param AnguloCamaras ngulo actual de las cmaras respecto al robot
 * @see IniMatrizCamaras()
 **************************************************************************/
void IniciarMatrices (matriz *giro,matriz *trasl,matriz *MatrizRobot,
					  matriz *matrizcamaras, Pt2D *PosicionActual,
					  double AnguloActual,double AnguloCamaras)
{
	matriz e,f;

	e=IniMatrix (3);	f=IniMatrix (3);

//	Calculamos las matrices necesarias para poscicionar y orientar el robot
//  en la imagen, y almacenamos la matriz resultante en "MatrizRobot"
	IniTraslacion((*trasl),(*PosicionActual).x,(*PosicionActual).y);
	IniGiro((*giro),AnguloActual);
	mulmat ((*giro),(*trasl),e);
//	Calculamos las matrices necesarias para poscicionar y orientar las cmaras
//  en la imagen, y almacenamos la matriz resultante en "matrizcamaras"
	IniMatrizCamaras(f,(*giro),(*trasl),AnguloCamaras);

	*MatrizRobot=e;	*matrizcamaras=f;
}

/**
 ***************************************************************************
 * Calcula el valor de la matriz necesaria para pasar los puntos de
coordenadas de las cmaras a coordenadas de mundo.
 * @param PosCamaras Matriz que contiene las transformaciones 2D necesarias para
pasar un punto de coordenadas de las camaras, a coordenadas de munfo
 * @param giro Matriz de giro del robot
 * @param traslacion Matriz de traslacin del robot
 * @param angulo ngulo (PAN) de las cmaras respecto al robot
 * @see IniciarMatrices()
 **************************************************************************/
void IniMatrizCamaras (matriz PosCamaras,matriz giro,matriz traslacion,double angulo)
{
	matriz GiroCamara,TraslCamara,tmp,tmp2,tmp3;
	tmp=IniMatrix(3);
	tmp2=IniMatrix(3);
	tmp3=IniMatrix(3);
	GiroCamara=IniMatrix(3);
	TraslCamara=IniMatrix(3);
	angulo=(angulo*PI)/180;

	// Inicializamon las matrices de giro y traslacion
	// de las camaras
	GiroCamara[0][0]=GiroCamara[1][1]=cos(angulo);
	GiroCamara[0][1]=sin(angulo);
	GiroCamara[1][0]=-sin(angulo);
	GiroCamara[2][2]=1;
	TraslCamara[0][0]=TraslCamara[1][1]=TraslCamara[2][2]=1;
	TraslCamara[2][0]=-69.86; //Coordenada X del centro de la cmara
	TraslCamara[2][1]=69.86;  //Coordenada Y del centro de la cmara

	// Comenzamos a multiplicar las matrices con el fin
	// de obtener la matriz de transformacion final
	mulmat(GiroCamara,TraslCamara,tmp);
	mulmat(tmp,giro,tmp2);
	mulmat(tmp2,traslacion,PosCamaras);

	FreeMatrix(tmp,3);
	FreeMatrix(tmp2,3);
	FreeMatrix(tmp3,3);
	FreeMatrix(GiroCamara,3);
	FreeMatrix(TraslCamara,3);
}

/**
 ***************************************************************************
 * Pinta el robot en el mapa. Teniendo en cuenta la posicin y orientacin
 del robot que define la matriz "MatrizRobot", se pinta el robot en el mapa
 * @param MatrizRobot Matriz que contiene todas las transformaciones necesarias
 para posicionar el robot en el mundo
 * @see PintaCamaras()
 * @see PintaZona()
 * @see CalculaPosRobot()
 **************************************************************************/
void PintaRobot (matriz MatrizRobot)
{
	Coordenadas=CalculaPosRobot (Robot,MatrizRobot);
	Coordenadas=CalculaPosRobot(Coordenadas,Base);
	MbufCopy (RefreshBuffer,WorkingBuffer);
	MgraColor(M_DEFAULT,0);
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.P1.x),long(Coordenadas.P1.y),long(Coordenadas.P2.x),long(Coordenadas.P2.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.P2.x),long(Coordenadas.P2.y),long(Coordenadas.P3.x),long(Coordenadas.P3.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.P3.x),long(Coordenadas.P3.y),long(Coordenadas.P4.x),long(Coordenadas.P4.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.P4.x),long(Coordenadas.P4.y),long(Coordenadas.P5.x),long(Coordenadas.P5.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.P5.x),long(Coordenadas.P5.y),long(Coordenadas.P6.x),long(Coordenadas.P6.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.P6.x),long(Coordenadas.P6.y),long(Coordenadas.P7.x),long(Coordenadas.P7.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.P7.x),long(Coordenadas.P7.y),long(Coordenadas.P8.x),long(Coordenadas.P8.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.P8.x),long(Coordenadas.P8.y),long(Coordenadas.P1.x),long(Coordenadas.P1.y));
	MgraColor(M_DEFAULT,M_RGB888(255,45,0));
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.BaseFlecha.x),long(Coordenadas.BaseFlecha.y),long(Coordenadas.PuntaFlechaC.x),long(Coordenadas.PuntaFlechaC.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.PuntaFlechaC.x),long(Coordenadas.PuntaFlechaC.y),long(Coordenadas.PuntaFlechaL.x),long(Coordenadas.PuntaFlechaL.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(Coordenadas.PuntaFlechaC.x),long(Coordenadas.PuntaFlechaC.y),long(Coordenadas.PuntaFlechaR.x),long(Coordenadas.PuntaFlechaR.y));

//	MgraDot(M_DEFAULT,RefreshBuffer,Coordenadas.centro.x,Coordenadas.centro.y);
//	MgraDot(M_DEFAULT,MilUnderlayBuffer,Coordenadas.centro.x,Coordenadas.centro.y);

	MgraColor(M_DEFAULT,0);
}

/**
 ***************************************************************************
 * Pinta las cmaras y su cono de visin en el mapa. Teniendo en cuenta la 
 posicin y orientacin de las cmaras que define la matriz "MatrizCamaras"
 se pintan las cmaras y su cono de visin.
 * @param MatrizRobot Matriz que contiene todas las transformaciones necesarias
 para posicionar las camaras y el cono de visin en el mundo
 * @see PintaRobot()
 * @see PintaZona()
 * @see CalculaPosCamaras()
  **************************************************************************/
void PintaCamaras (matriz MatrizCamaras)
{
	vision coord;

	coord=CalculaPosCamaras(camaras,MatrizCamaras);
	coord=CalculaPosCamaras(coord,Base);

	MgraLine(M_DEFAULT,WorkingBuffer,long(coord.uno.x),long(coord.uno.y),long(coord.dos.x),long(coord.dos.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(coord.dos.x),long(coord.dos.y),long(coord.tres.x),long(coord.tres.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(coord.tres.x),long(coord.tres.y),long(coord.cuatro.x),long(coord.cuatro.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(coord.cuatro.x),long(coord.cuatro.y),long(coord.uno.x),long(coord.uno.y));
	MgraColor(M_DEFAULT,M_RGB888(103,162,249));
	MgraLine(M_DEFAULT,WorkingBuffer,long(coord.uno.x),long(coord.uno.y),long(coord.Cono1L.x),long(coord.Cono1L.y));
	MgraLine(M_DEFAULT,WorkingBuffer,long(coord.uno.x),long(coord.uno.y),long(coord.Cono1R.x),long(coord.Cono1R.y));
	MgraColor(M_DEFAULT,0);
}

/**
 ***************************************************************************
 * Pinta la zona de alta resolucin.  Pinta la zona de alta resolucin el mapa
 si asi lo determina la estructura "ZonaAltaDensidad"
 * @param ZonaAltaDensidad Estructura que contiene la informacin necesaria
 para pintar la zona de alta resolucin
 * @see PintaRobot()
 * @see PintaCamaras()
 **************************************************************************/
void PintaZona (zona ZonaAltaDensidad)
{
	if (ZonaAltaDensidad.UsarZona==true)  {
		Pt2D	P1,P2,P3,P4;
		Pt2D	temp1,temp2;

		temp1.x=ZonaAltaDensidad.EsqInfDrch.x;
		temp1.y=ZonaAltaDensidad.EsqSupIzq.y;
		temp2.x=ZonaAltaDensidad.EsqSupIzq.x;
		temp2.y=ZonaAltaDensidad.EsqInfDrch.y;
		P1=MultVect(Base,ZonaAltaDensidad.EsqSupIzq);
		P2=MultVect(Base,ZonaAltaDensidad.EsqInfDrch);
		P3=MultVect(Base,temp1);
		P4=MultVect(Base,temp2);
		MgraColor(M_DEFAULT,M_RGB888(1,215,92));
		MgraRect(M_DEFAULT,WorkingBuffer,P1.x,P1.y,P2.x,P2.y);
		MgraColor(M_DEFAULT,0);
	}
}

/**
 ***************************************************************************
 * Transforma las coordenadas de los puntos que definen el robot virtual. 
 Transforma las coordenadas de los puntos que definen el contorno del robot 
 virtual, en funcin de una matriz de transformacin "matrix"
 * @param robot Contiene los puntos del contorno del robot virtual en coordenadas
 de la base solidaria al robot.
 * @param matrix Matriz que contiene las transformaciones que se le aplicarn
 a los punto del contorno del robot virtual.
 * @return Los puntos de contorno del robot virtual transformados.
 * @see CalculaPosCamaras()
 **************************************************************************/
posicion CalculaPosRobot (posicion robot,matriz matrix)
{
	posicion temp;
	temp.P1=MultVect(matrix,robot.P1);
	temp.P2=MultVect(matrix,robot.P2);
	temp.P3=MultVect(matrix,robot.P3);
	temp.P4=MultVect(matrix,robot.P4);
	temp.P5=MultVect(matrix,robot.P5);
	temp.P6=MultVect(matrix,robot.P6);
	temp.P7=MultVect(matrix,robot.P7);
	temp.P8=MultVect(matrix,robot.P8);
	temp.centro=MultVect(matrix,robot.centro);
	temp.BaseFlecha=MultVect(matrix,robot.BaseFlecha);
	temp.PuntaFlechaC=MultVect(matrix,robot.PuntaFlechaC);
	temp.PuntaFlechaL=MultVect(matrix,robot.PuntaFlechaL);
	temp.PuntaFlechaR=MultVect(matrix,robot.PuntaFlechaR);
	return temp;
}

/**
 ***************************************************************************
 * Transforma las coordenadas de los puntos que definen las camaras. 
 Transforma las coordenadas de los puntos que definen la forma de las cmaras y 
 su cono de visin, en funcin de una matriz de transformacin "matrix"
 * @param camara Contiene los puntos que definen la forma de las camaras y su cono
 de visin, en coordenadas de la base solidaria a las camaras.
 * @param matrix Matriz que contiene las transformaciones que se le aplicarn
 a los punto que definen las cmaras y su cono de visin.
 * @return Los puntos que definen las cmaras y su cono de visin transformados.
 * @see CalculaPosRobot()
 **************************************************************************/
vision CalculaPosCamaras (vision camara, matriz matrix)
{
	vision temp;
	temp.uno=MultVect(matrix,camara.uno);
	temp.dos=MultVect(matrix,camara.dos);
	temp.tres=MultVect(matrix,camara.tres);
	temp.cuatro=MultVect(matrix,camara.cuatro);
	temp.Cono1L=MultVect(matrix,camara.Cono1L);
	temp.Cono1R=MultVect(matrix,camara.Cono1R);
	return temp;
}

/**
 ***************************************************************************
 * Actualiza la lista de lecturas de sonar a representar en el mapa.
 Recopila la informacin de las lecturas de los sonares contenida en los 3 buffers
 de Saphira, cambia sus coordenadas para que esten en coordenadas de la base
 solidaria al robot, y las almacena en la lista de lecturas de sonar a representar
 en el mapa.
 * @see BuscarLectura()
 **************************************************************************/
void s_actualitzar (void)
{
	int		limit, i;
	Pt2D	Punto,PuntoMundo;
	bool	ValorRepetido=false;

	i = 0;
	SonarBufferIndex = 0;
	limit = sr_buf->limit;
	while (i<limit)
	{
		if (sr_buf->valid[i])
		{
			SonarBuffer[SonarBufferIndex].x = sr_buf->xbuf[i];
			SonarBuffer[SonarBufferIndex].y = sr_buf->ybuf[i];
			SonarBufferIndex++;
		}
		i++;
	}

	i = 0;
	limit = sl_buf->limit;
	while (i<limit)
	{
		if (sl_buf->valid[i])
		{
			SonarBuffer[SonarBufferIndex].x = sl_buf->xbuf[i];
			SonarBuffer[SonarBufferIndex].y = sl_buf->ybuf[i];
			SonarBufferIndex++;
		}
		i++;
	}

	i = 0;
	limit = sraw_buf->limit;
	while (i<limit)
	{
		if (sraw_buf->valid[i])
		{
			SonarBuffer[SonarBufferIndex].x = sraw_buf->xbuf[i];
			SonarBuffer[SonarBufferIndex].y = sraw_buf->ybuf[i];
			SonarBufferIndex++;
		}
		i++;
	}

	i=0;
	while (i<SonarBufferIndex)

	{
		Punto.x = -SonarBuffer[i].y;  //estos cambio son devidos a las
		Punto.y = SonarBuffer[i].x;	  //diferentes referencias
		PuntoMundo=MultVect(MatrizRobot,Punto);
		i++;
		ValorRepetido=BuscarLectura(PuntoMundo);
		if (ValorRepetido==false)
		{
			ListaFIFO[Indice1]=PuntoMundo;
			Indice1++;
		}
		if (Indice1>=LISTA_MAX) {Indice1=0;}
	}
}

/**
 ***************************************************************************
 * Comprueba si un punto concreto existe en la lista de lecturas de sonar.
 * @param Valor Punto en coordenadas de mundo que se pretende comprobar si
 existe en la lista de lecturas de sonar a representar en el mapa
 * @return TRUE si existe en la lista, FALSE si no.
 **************************************************************************/
bool BuscarLectura (Pt2D Valor)
{
	int i;

	for (i=0;i<LISTA_MAX;i++)
	{
		if (ListaFIFO[i].x==Valor.x && ListaFIFO[i].y==Valor.y) {
			return true;
		}
	}
	return false;
}

/**
 ***************************************************************************
 * Lee la informacin de un fichero de mundo.  Lee la informacin contenida
 en un fichero de mundo y la almacena en memoria.
 * @param objetos Apuntador a la estructura de datos que almacenar en memoria la 
 informacin de mundo contenida en el fichero.
 * @param Mapa Path completo del fichero de mundo que se intenta leer
 * @param DirectorioObjetos Path del directorio donde se encuentran los
 ficheros que definen los objetos que se encuentran en el mapa
 * @param lugar Devuelve el punto de mundo en que se supone que estar el 
 robot al cargar el mapa
 * @param Horientacion Devuelve la orientacin que se supone que tendr el 
 robot al cargar el mapa
 * @see AbrirMapaMIL()
 * @see BorrarMundo()
 * @see AnadirNodo()
  **************************************************************************/
void LeerMundo (lista *objetos,char *Mapa,char *DirectorioObjetos,Pt2D *lugar
				,double *Horientacion)
{
//	char directorio[100];
	char fichero[100];
	int m,n,i,longitud,NumeroObjetos;
	char nombre[100];
	char aux[100];
	char *nombre2,*codigo;
	bool ObjetoMovil=false;
	long x,y,NumeroPuntos;
	Pt2D punto;
	double PosX,PosY,heading;

	map=fopen(Mapa,"r");
	fscanf(map,"%d",&NumeroObjetos);	//Obtenemos el numero de objetos del mundo

	for (i=0;i<NumeroObjetos;i++)
	{
		for (m=0;objetos[m]!=NULL;m++); // buscamos el primer objeto de la lista
										// que no este definido y por tanto sea NULL
		fscanf(map,"%s",&fichero);
		strcat(fichero,".dat");
		sprintf (aux,"%s%s",DirectorioObjetos,fichero);	
		in=fopen(aux,"r");

		fscanf(in,"%s",&nombre);
		longitud=strlen(nombre);
		nombre2=(char *)calloc(longitud,sizeof(char));
		strcpy(nombre2,nombre);
	
		fscanf(in,"%s",&nombre);
		longitud=strlen(nombre);
		codigo=(char *)calloc(longitud,sizeof(char));
		strcpy(codigo,nombre);

		if (strncmp(codigo,"001",3)!=0) {ObjetoMovil=true;}

		fscanf(in,"%d",&NumeroPuntos);	//Obtenemos el nunero de puntos que 
										// tiene el contorno
		for (n=0;n<NumeroPuntos;n++)
		{
			fscanf(in,"%d %d",&x,&y);
			punto.x=x; punto.y=y;
			objetos[m]=AnadirNodo(objetos[m],punto);
		}
		objetos[m]->nombre=nombre2;
		objetos[m]->codigo=codigo;
		fclose(in);

		if (ObjetoMovil==true)	//si el objeto es movil tenemos que coger
		{						//su posicion y horientacion
			fscanf (map,"%lf %lf %lf",&PosX,&PosY,&heading);
			punto.x=PosX; punto.y=PosY;
			objetos[m]->Posicion =punto;
			objetos[m]->Horientacion =heading;
			ObjetoMovil=false;
		}
	}
//	Leemos la poscion y horientacin que tiene que tener el robot en el mapa
	fscanf (map,"%lf %lf %lf",&PosX,&PosY,&heading);
	(*lugar).x=PosX;    (*lugar).y=PosY;
	*Horientacion=heading;
	fclose(map);
}

/**
 ***************************************************************************
 * Desalocata de memoria el mundo almacanado
 * @param objetos Apuntador al mundo que hay en memoria y que se pretende
 desalocatar
 * @see BorrarLista()
 **************************************************************************/
void BorrarMundo (lista *objetos)
{
	int			i=0;
	while (objetos[i]!=NULL)
	{	
		BorrarLista (objetos[i]);
		objetos[i]=NULL;
		i++;
	}
}

/**
 ***************************************************************************
 * Reserva espacio de memoria para un objeto de mundo.
 * @return El apuntador al espacio de memoria reservado para el objeto
 * @see BorrarLista()
 **************************************************************************/
lista CrearLista () 
{
	return (lista)malloc(sizeof(poligono));
}

/**
 ***************************************************************************
 * Libera el espacio de memoria ocupado por un objeto de mundo concreto
 * @param Lista_a_Borrar Objeto de mundo que se pretende desalocatar
 * @see CrearLista()
 * @see BorrarNodo()
 **************************************************************************/
void BorrarLista (lista Lista_a_Borrar)
{
	nodo_ptr	actual,siguiente;

	actual=Lista_a_Borrar->ultimo;
	while (actual!=NULL)
	{
		siguiente=actual->next;
		BorrarNodo(actual);
		actual=siguiente;
	}
	free (Lista_a_Borrar);
}

/**
 ***************************************************************************
 * Aade un nodo al contorno que define un objeto de mundo.  
 Esta funcin reserva espacio de memoria para un nodo, y lo aade al contorno 
 de un objeto mundo concreto. Si el objeto de mundo no existe, se crea y 
 se le asigna como primer punto del contorno, el nodo que se pretenda alocatar.
 * @param l Objeto de mundo a cuyo contorno se le pretende aadir un punto
 * @param valor Punto en cooordenadas de mundo que se pretende aadir al 
 contorno de un objeto
 * @return Un objeto de mundo a cuyo contorno se le ha aadido el nodo.
 * @see CrearNodo()
 * @see BorrarNodo()
 **************************************************************************/
lista AnadirNodo (lista l,Pt2D valor) 
{	
	nodo_ptr nodo;
	lista temp;

	nodo=CrearNodo();

	if (l==NULL){
		temp=CrearLista();
		l=temp;
		l->ultimo=nodo;
		l->ultimo->next=NULL;
		l->ultimo->value=valor;
	}
	else {
		nodo->next=l->ultimo;
		nodo->value=valor;
		l->ultimo=nodo;
	}
	return l;
}

/**
 ***************************************************************************
 * Reserva espacio de memoria para un nodo
 * @return Un apuntador al nodo alocatado
 * @see BorrarNodo()
 * @see AnadirNodo()
 **************************************************************************/
nodo_ptr CrearNodo () 
{
	return (nodo_ptr)malloc(sizeof(nodo));
}

/**
 ***************************************************************************
 * Libera el espacio de memoria ocupado por un nodo concreto
 * @param Nodo_a_Borrar Nodo que se pretende desalocatar
 * @see CrearNodo()
 * @see AnadirNodo()
 **************************************************************************/
void BorrarNodo (nodo_ptr Nodo_a_Borrar)
{
	free (Nodo_a_Borrar);
}

/**
 ***************************************************************************
 * Pinta el mapa del mundo
 * @param objetos Apuntador a la estructura de datos que contiene la informacin
 del mundo que se pretende pintar
 * @param modo Manera en que se pintarn los objetos del mundo
 * @see PintarObjeto()
 **************************************************************************/
void PintarMundo (lista *objetos,int modo)
{
	int m=0;

	while (objetos[m]!=NULL)
	{
		PintarObjeto(objetos[m],modo);
		m++;
	}
//	MgraArc (M_DEFAULT,WorkingBuffer,
}

/**
 ***************************************************************************
 * Pinta un objeto en el plano.
 * @param Objeto Objeto a representar en el mundo
 * @param modo Manera en que se representan los objetos en el plano.  Si modo=0=MOVIMIENTO
 entonces el contorno de la habitacin y los objetos no se rellenan; si modo=1=ESTACIONARIO
 los objetos y el contorno de la habitacin se rellenan de un color.
 * @see PintarMundo()
 * @see RellenarObjeto()
 **************************************************************************/
void PintarObjeto (lista objeto,int modo)
{
	nodo_ptr tmp;
	Pt2D anterior,actual,PuntoMundo;
	lista ObjetoMundo=NULL;

	if (strncmp(objeto->codigo,"001",3)==0)
	{
		if (modo==ESTACIONARIO) {		// En caso que pintemos el mundo para estar quieto
			MbufClear(WorkingBuffer,0);	// pintamos el fondo negro y
			MgraColor (M_DEFAULT,255);	// el contorno de blanco
		}
		else { 
			MbufClear(WorkingBuffer,255);	// si no pintamos el fondo blanco y
			MgraColor (M_DEFAULT,0);		// el contorno negro
		}

		tmp=objeto->ultimo;
		anterior=MultVect(Base,tmp->value);	
		ObjetoMundo=AnadirNodo(ObjetoMundo,anterior);
		tmp=tmp->next;
		while (tmp!=NULL){
			actual=MultVect(Base,tmp->value);
			ObjetoMundo=AnadirNodo(ObjetoMundo,actual);
			MgraLine (M_DEFAULT, WorkingBuffer, anterior.x, anterior.y, actual.x, actual.y);
			anterior=actual;
			tmp=tmp->next;
		}
		actual=MultVect(Base,objeto->ultimo->value);
		ObjetoMundo=AnadirNodo(ObjetoMundo,actual);
		MgraLine (M_DEFAULT, WorkingBuffer, anterior.x, anterior.y, actual.x, actual.y);

		// rellenamos el objeto de blanco porque es el contorno de la habitacion
		if (modo==ESTACIONARIO){ RellenarObjeto (ObjetoMundo,WorkingBuffer,255);}
	}
	else if (strncmp(objeto->codigo,"002",3)==0)
	{
		//	Calculamos la matriz de posicion y horientacion del objeto en cuestion
		IniTraslacion(Traslacion,objeto->Posicion.x,objeto->Posicion.y);
		IniGiro(Giro,objeto->Horientacion);
		mulmat (Giro,Traslacion,MatrizObjetos);

		tmp=objeto->ultimo;
		MgraColor (M_DEFAULT,100);

		PuntoMundo=MultVect(MatrizObjetos,tmp->value);
		anterior=MultVect(Base,PuntoMundo);
		ObjetoMundo=AnadirNodo(ObjetoMundo,anterior);
		tmp=tmp->next;

		while (tmp!=NULL){
			PuntoMundo=MultVect(MatrizObjetos,tmp->value);
			actual=MultVect(Base,PuntoMundo);
			ObjetoMundo=AnadirNodo(ObjetoMundo,actual);
			MgraLine (M_DEFAULT, WorkingBuffer, anterior.x, anterior.y, actual.x, actual.y);
			anterior=actual;
			tmp=tmp->next;
		}
		PuntoMundo=MultVect(MatrizObjetos,objeto->ultimo->value);
		actual=MultVect(Base,PuntoMundo);
		ObjetoMundo=AnadirNodo(ObjetoMundo,actual);
		MgraLine (M_DEFAULT, WorkingBuffer, anterior.x, anterior.y, actual.x, actual.y);

		// rellenamos el objeto pero de negro
		if (modo==ESTACIONARIO){ 
			RellenarObjeto (ObjetoMundo,WorkingBuffer,0);
		}
	}
	BorrarLista(ObjetoMundo);
}

/**
 ***************************************************************************
 * Pinta el interior de un objeto de un color determinado.  Esta funcin busca 
 un punto interno a un objeto, y lo utiliza como punto semilla para rellenar
 el objeto de un color determinado.
 * @param objeto Objeto del mundo a rellenar
 * @param buffer Buffer en el que se representa la escena (mapa)
 * @param color Entero que representa el color con el que se rellenar el objeto
 * @see PintarObjeto()
 * @see Punto_en_Buffer()
 **************************************************************************/
void RellenarObjeto (lista objeto,MIL_ID buffer,int color)
{
	polygon_ptr poligono;
	nodo_ptr tmp;
	long punto_x=0,punto_y=0;
	long Tolerancia=10;
	Pt2D punto;
	vtx_ptr new_vertex;
	pt_poly_relation posicion_del_punto=FUERA;

/* COPIAMOS LOS DATOS DE LA ESTRUCTURA DE LISTA A LA 
	ESTRUCTURA USADA POR EL ALGORITMO POINT_IN_POLY */
	polygon_create(poligono);

	tmp=objeto->ultimo;
	while (tmp!=NULL){
		polygon_new_vertex(poligono,tmp->value.x,tmp->value.y,new_vertex);
		tmp=tmp->next;
	}

/* MIRAMOS LOS 8 VECINOS DE CADA NODO HASTA ENCONTRAR 
	UN PUNTO INTERIOR AL POLIGONO */
	while (Tolerancia!=NULL && posicion_del_punto==FUERA){
		tmp=objeto->ultimo;
		while (posicion_del_punto==FUERA && tmp!=NULL){
			punto_x=tmp->value.x+Tolerancia;
			punto_y=tmp->value.y;
			posicion_del_punto=point_in_poly(poligono,punto_x,punto_y);
			if (posicion_del_punto==FUERA){
				punto_y=punto_y+Tolerancia;
				posicion_del_punto=point_in_poly(poligono,punto_x,punto_y);
				if (posicion_del_punto==FUERA){
					punto_x=punto_x-Tolerancia;
					posicion_del_punto=point_in_poly(poligono,punto_x,punto_y);
					if (posicion_del_punto==FUERA){
						punto_x=punto_x-Tolerancia;
						posicion_del_punto=point_in_poly(poligono,punto_x,punto_y);
						if (posicion_del_punto==FUERA){
							punto_y=punto_y-Tolerancia;
							posicion_del_punto=point_in_poly(poligono,punto_x,punto_y);
							if (posicion_del_punto==FUERA){
								punto_y=punto_y-Tolerancia;
								posicion_del_punto=point_in_poly(poligono,punto_x,punto_y);
								if (posicion_del_punto==FUERA){
									punto_x=punto_x+Tolerancia;
									posicion_del_punto=point_in_poly(poligono,punto_x,punto_y);
									if (posicion_del_punto==FUERA){
										punto_x=punto_x+Tolerancia;
										posicion_del_punto=point_in_poly(poligono,punto_x,punto_y);
									}
								}
							}
						}
					}
				}
			}

			tmp=tmp->next;
			punto.x=punto_x; punto.y=punto_y;
			if (!Punto_en_Buffer(punto)) posicion_del_punto=FUERA;
		}
		if (posicion_del_punto==FUERA){	//Este SWITCH se encarga de 
			switch (Tolerancia)			//afinar la tolerancia en caso
			{							//de no encontrar ningun
			case 10:					//punto interno despues de
				Tolerancia=2;			//repasar todo los puntos 
				break;					//del polgono
			case 2:
				Tolerancia=NULL;
			}
		}
	}
	if (posicion_del_punto==DENTRO) {
		MgraColor(M_DEFAULT,(double)color);
		MgraFill(M_DEFAULT,buffer,punto.x,punto.y);
/*		MgraColor(M_DEFAULT,M_RGB888(50,0,0));
		MgraArc (M_DEFAULT,buffer,punto.x,punto.y,10,10,0,350);
		MgraColor(M_DEFAULT,(double)color);*/
	}
}

/**
 ***************************************************************************
 * Comprueba si un punto esta dentro de un buffer.
 * @param Punto Punto que se somete a estudio
 * @return TRUE si el punto se encuentra dentro del buffer, FALSE si no.
 **************************************************************************/
bool Punto_en_Buffer (Pt2D Punto)
{
	if (Punto.x>=0 && Punto.x<=Buffer_Width)
	{
		if (Punto.y>=0 && Punto.y<=Buffer_Height) return TRUE;
	}
	return FALSE;
}