#include "malloc.h"
#include "2Dlib.h"
#include "saphira.h"
#include "process.h"
#include "pt_poly.h"
#include "mil.h"
#include "conio.h"

#define DLL_EXPORT extern "C" __declspec(dllexport)
#define DLL_IMPORT extern "C" __declspec(dllimport)
#define STD __stdcall
#define PI					3.141592
#define LISTA_MAX			100
#define CHILDBUFFER_WIDTH	210
#define CHILDBUFFER_HEIGHT	210
#define MOVIMIENTO			0
#define	ESTACIONARIO		1

typedef double** matriz;

FILE *in,*map;

////////////////////////////////////////////////////////////
//    Definicion de las estructuras 
////////////////////////////////////////////////////////////

/**
****************************************************************
*  Estructura de datos que contiene la informacin del contorno 
del robot virtual.  Esta estructura de datos contiene los puntos que 
definen la forma robot virtual.  Los puntos se definen en milmetros y en 
coordenadas de la base solidaria al robot virtual
****************************************************************/
typedef struct {
	Pt2D P1;
	Pt2D P2;
	Pt2D P3;
	Pt2D P4;
	Pt2D P5;
	Pt2D P6;
	Pt2D P7;
	Pt2D P8;
	Pt2D centro;
	Pt2D BaseFlecha;
	Pt2D PuntaFlechaC;
	Pt2D PuntaFlechaL;
	Pt2D PuntaFlechaR;
}posicion;

/**
****************************************************************
*  Estructura de datos que contiene la informacin para representar
el cono de visin de las cmaras.  Esta estructura de datos contiene, 
los puntos que definen la forma con la que se representan las cmaras
y su cono de visin en el mapa del Navegador.  Los puntos se definen en 
milmetros y en coordenadas de la base solidaria a las cmaras (su representacin
grfica)
****************************************************************/
typedef struct {
	Pt2D uno;
	Pt2D dos;
	Pt2D tres;
	Pt2D cuatro;
	Pt2D Cono1L;
	Pt2D Cono1R;
}vision;

/**
****************************************************************
*  Estructura de datos que define la ventana de resolucin.  Esta estructura
contiene la informacin de la esquina superior izquierda y la esquina inferior
derecha, que definen la ventana de resolucin en el mapa del Navegador.  Las esquinas
de la ventana de resolucin estan definidas en coordenadas de Mundo.
Si su campo "UsarZona" no es verdadero, aunque se llame a la funcion que 
pinta la ventana de resolucin, esta no se pintar
****************************************************************/
typedef struct {
	/** Esquina superior izquierda de la ventna */
	Pt2D EsqSupIzq;
	/** Esquina inferior derecha de la ventana */
	Pt2D EsqInfDrch;
	/** Este Boolean determina si se ha de representar la ventana
	de resolucin en el mapa */
	bool UsarZona;
}zona;

/**
****************************************************************
*  Estructura de datos que contiene la informacin de uno de los 
puntos del contorno de un objeto de mundo.
Esta estructura almacena uno de los puntos del contorno de un objeto
de mundo, y un apuntador al siguiente nodo del contorno. Si es
el ltimo punto del contorno del objeto, el apuntador al siguiente nodo
apunta a NULL
****************************************************************/
typedef struct nodo{
	/** Apuntador al siguiente nodo del contorno del objeto*/
	struct nodo *next;
	/** Punto del contorno del objeto */
	Pt2D value;	
}nodo;

typedef nodo *nodo_ptr;


/**
************************************************************************
* Estructura de datos que contiene la informacin de un objeto de mundo
***********************************************************************/
typedef struct {
	/**Nombre del objeto*/
	char *nombre; 	
	/**Codigo del objeto*/
	char *codigo;	
	/**Posicin del objeto en el mundo*/
	Pt2D Posicion;  
	/**Orientacin del objeto en el mundo*/
	double Horientacion;
	/**Apuntador al primer nodo del contorno del objeto*/
	nodo_ptr ultimo;	
}poligono;

typedef poligono *lista;

////////////////////////////////////////////////////////////
//		Variables Globales
///////////////////////////////////////////////////////////

MIL_ID		BufferBackground, MilUnderlayBuffer;
MIL_ID		disp2,temp;
MIL_ID		WorkingDisplay,WorkingBuffer;
MIL_ID		RefreshDisplay,RefreshBuffer;
MIL_ID		SonaresDisplay,SonaresBuffer;
matriz		Giro,Traslacion,Simetria_X,Base,InversaBase;
matriz		MatrizRobot,MatrizCamaras;
matriz		MatrizObjetos;
Pt2D		PosicionActual;
posicion	Robot,Coordenadas;
vision		camaras;
zona		ZonaAltaDensidad;
bool		PararSonares=false;
double		ESCALA_PixPorMM, AlturaImagen;
///long        New_PosX, New_PosY;
long		TransparentColor;
long		Buffer_Width,Buffer_Height;

//	Variables necesarias para la lectura de los sonares
Pt2D		SonarBuffer[3*CBUF_LEN];
int			SonarBufferIndex;

Pt2D		ListaFIFO[LISTA_MAX];
int			Indice1=0,Indice2=0;

Pt2D		old_sonar_lectures[LISTA_MAX];
int			IndiceBorrado1=0, IndiceBorrado2=0;
bool		BorrarLecturasViejas=false;

//	Variables del mundo
lista		Mundo[30];

//////////////////////////////////////////////////////////////
//		Cabeceras de las funciones
/////////////////////////////////////////////////////////////

DLL_EXPORT void STD AbrirMapaMIL (HWND WindowHandle,char *FICHERO,char *DIRECTORIO_OBJETOS, 
								  long *alto,long *ancho,double escala,double *PosX,double *PosY,
								  double *Heading,MIL_ID sys,MIL_ID *disp,MIL_ID *mapa);

DLL_EXPORT void STD NuevoMapaMIL (HWND WindowHandle,long alto, long ancho ,
								  double escala,MIL_ID sys,MIL_ID *disp,
								  MIL_ID *mapa);

DLL_EXPORT void STD SalvarMapaMIL (char *Filename);

DLL_EXPORT void STD RecalcularMapaMIL (double escala,double DesplazamientoH,
									   double DesplazamientoV,double AnguloGiro,
									   long EjeGiroX,long EjeGiroY,int modo);

DLL_EXPORT void STD CerrarMapaMIL (MIL_ID disp);

DLL_EXPORT void STD UsarZonaAltaResolucion (void);

DLL_EXPORT void STD AjustarZonaAltaResolucion (long XSupIzq,long YSupIzq, 
										   long XInfDrch,long YInfDrch);

DLL_EXPORT void STD NoUsarZonaAltaResolucion (void);

DLL_EXPORT void STD PosicionarRobotVirtual (double x, double y, double Angulo,
											double AnguloCamaras, long modo);

DLL_EXPORT void STD ActualizarVista (double x, double y, double Angulo
									   ,double AnguloCamaras, long modo);

DLL_EXPORT void STD ObtenerLecturaSonares (void);

DLL_EXPORT void STD VaciarBufferLecturasSonares (void);

//DLL_EXPORT void STD Tratar_Lecturas_Sonares (void);

DLL_EXPORT void STD TransfMundoDispositivo (double PosXMundo, double PosYMundo,
											double *PosXDisp, double *PosYDisp);

DLL_EXPORT void STD TransfDispositivoMundo (double PosXDisp, double PosYDisp,
											double *PosXMundo, double *PosYMundo);

DLL_EXPORT void STD SplineTracer (double x1, double y1, double x2, double y2);

DLL_EXPORT void STD PintarPuntoTrayectoria (double x1, double y1);

//	FUNCIONES DE BAJO NIVEL

matriz IniciarMatrizCambioBase (matriz *giro,matriz *trasl,matriz *simetria_x
									 ,double AlturaImagen, double DesplazamientoH,
									 double DesplazamientoV,double AnguloGiro,
									 long EjeGiroX,long EjeGiroY);

void IniciarMatrices (matriz *giro,matriz *trasl,matriz *MatrizRobot,
					  matriz *matrizcamaras, Pt2D *PosicionActual,
					  double AnguloActual,double AnguloCamaras);

void IniMatrizCamaras (matriz PosCamaras,matriz giro,matriz traslacion,double angulo);
void PintaRobot (matriz MatrizRobot);
void PintaCamaras (matriz MatrizCamaras);
void PintaZona (zona ZonaAltaDensidad);
posicion CalculaPosRobot (posicion robot,matriz matrix);
vision CalculaPosCamaras (vision camara, matriz matrix);
void s_actualitzar (void);
bool BuscarLectura (Pt2D Valor);
void BorrarRobot (void);
void ComprobarPosChildBuffer (void);
void LeerMundo (lista *objetos,char *Mapa,char *DirectorioObjetos,Pt2D *lugar
				,double *Horientacion);
void BorrarMundo (lista *objetos);
lista CrearLista ();
void BorrarLista (lista Lista_a_Borrar);
lista AnadirNodo (lista l,Pt2D valor);
void BorrarNodo (nodo_ptr Nodo_a_Borrar);
nodo_ptr CrearNodo ();
void PintarMundo (lista *objetos,int modo);
void PintarObjeto (lista objeto,int modo);
void RellenarObjeto (lista objeto,MIL_ID buffer,int color);
bool Punto_en_Buffer (Pt2D Punto);

DLL_EXPORT void STD GrabarBuffer ();
