#include "robot.h"

/**
 **************************************************************************** 
 * Realiza la conexin con el robot
 * @param TipoConexion Decide a quien conecto 0=Simulador 1=RobotReal
 * @param XPosInic Posicin inicial que ocupar en robot en el mapa (eje X)
 * @param YPosInic Posicin inicial que ocupar en robot en el mapa (eje Y) 
 * @param Heading Orientacin que tendra el robot
 * @param DireccionIP Direccin IP del robot real
 * @see DesconexionRobot()
 * @return Devuelve un 1 si ha conectado y un 0 si no lo ha hecho
 ***************************************************************************/
DLL_EXPORT long STD ConexionConRobot (long TipoConexion, double XPosInic ,
									  double YPosInic, double Heading, char *DireccionIP)
{
	long i;
	switch (TipoConexion)
	{
	case 0:
		i=paiRobotStartup (sfLOCALPORT,sfCOMLOCAL,NULL,NULL);
		break;
	case 1:
		i=paiRobotStartup (TCP,DireccionIP,NULL,NULL);
		break;
	}
	if (i==1) {
		sfRobotComInt(4,1);	//Conecta los motores del robot si hay conexion con l
		sfMoveRobot ((float)YPosInic,-(float)XPosInic,(float)Heading);  //Comunica a Saphira la posicin que  
														   //ocupa el robot en el momento de conectarlo
	}
	return i;
}

/**
 **********************************************************
 * Desconecta del robot, tanto del real como del simulador
 * @see ConexionConRobot()
 **********************************************************/
DLL_EXPORT void STD DesconexionRobot (void)
{
	sfDisconnectFromRobot();
}

int paiRobotStartup(int channel, char *name, HANDLE hInst, int nCmdShow)
{
	int i;
	sfOnConnectFn(myConnectFn);	/* register a connection function */
	sfOnStartupFn(myStartupFn);	/* register a startup function */
	sfStartup(hInst, nCmdShow, 1);	/* start up Saphira, and  keep going */
	i = 50;
	while (!sfIsConnected && i--) {
		sfPause(50);
	}
	sfInitControlProcs();
	sfRunEvaluator();
	return sfConnectToRobot(channel, name); 
}

void myStartupFn(void)
{
	sfSetDisplayState(sfGLOBAL, TRUE);	/* use the global view */
}

void myConnectFn(void)
{
	sfInitBasicProcs();		/* start up comm processes */
}	

/**
 ***********************************************************************
 * Hace girar el robot en el sentido de las agujas del reloj a
velocidad constante.
 * @param Velocidad Velocidad de giro en grados por segundo
 * @see GirarRobotIzquierda()
 * @see DetenerGiroRobot()
 * @see GirarRobotHastaAnguloN()
 ***********************************************************************/
DLL_EXPORT void STD GirarRobotDerecha (long velocidad)
{
	sfSetRVelocity((int)(-velocidad));
}

/**
 ************************************************************************
 * Hace girar el robot en sentido opuesto al de las agujas del reloj a
velocidad constante.
 * @param Velocidad Velocidad de giro en grados por segundo
 * @see GirarRobotDerecha()
 * @see DetenerGiroRobot()
 * @see GirarRobotHastaAnguloN()
 **********************************************************************/
DLL_EXPORT void STD GirarRobotIzquierda (long velocidad)
{
	sfSetRVelocity((int)(velocidad));
}
/**
 *****************************************************************************
 * Gira el robot hasta el angulo absoluto N
 * @param Angulo Angulo hasta el que gira el robot. Debe estar entre 0 y 359
 * @see GirarRobotDerecha()
 * @see GirarRobotIzquierda()
 * @see DetenerGiroRobot()
 ****************************************************************************/
DLL_EXPORT void STD GirarRobotHastaAnguloN (double Angulo)
{
	sfSetHeading ((int)(Angulo));
}

/**
 ***************************************************************************
 * Detiene la rotacin del robot fijando la velocidad de rotacin a cero.
 * @see GirarRobotDerecha()
 * @see GirarRobotIzquierda()
 * @see GirarRobotHastaAnguloN()
 ***************************************************************************/
DLL_EXPORT void STD DetenerGiroRobot (void)
{
//	do {
		sfSetRVelocity(0);
//	}while (sfRobot.rv!=0);
}

/**
 *****************************************************************************
 * Hace avanzar el robot a velocidad constante
 * @param Velocidad Velocidad de traslacin en mm/seg
 * @see RetrocederRobot
 * @see DetenerTraslacionRobot
 ****************************************************************************/
DLL_EXPORT void STD AvanzarRobot (long velocidad)
{
	sfSetVelocity(velocidad);
}

/**
 **************************************************************************
 * Hace retroceder el robot a velocidad constante
 * @param Velocidad Velocidad de traslacin en mm/seg
 * @see AvanzarRobot
 * @see DetenerTraslacionRobot
 *************************************************************************/
DLL_EXPORT void STD RetrocederRobot (long velocidad)
{
	sfSetVelocity(-velocidad);
}

/**
 **************************************************************************
 * Detine la traslacin del robot fijando su velocidad de traslacin a cero
 * @see AvanzarRobot
 * @see RetrocederRobot
 *************************************************************************/
DLL_EXPORT void STD DetenerTraslacionRobot (void)
{
	sfSetVelocity(0);
}

/**
 ***************************************************************************
 * Devuelve la posicin y orientacin absolutas del robot
 * @param PosX Devuelve la coordenada X de la posicin absoluta del robot
 * @param PosY Devuelve la coordenada Y de la posicin absoluta del robot
 * @param Heading Devuleve Orientacin absoluta del robot
 **************************************************************************/
DLL_EXPORT void STD GetRobotPosition (double *PosX,double *PosY,double *Heading)
{
	*PosX=-sfRobot.ay;
	*PosY=sfRobot.ax;
	*Heading=sfRobot.ath;
}

/**
 *****************************************************************************************************
 * Esta funcion carga los comportamientos predefinidos por Saphira
 * @param Parametros Puntero a una estructura que contiene toda la informacion de los comportamientos
 * @see DescargarBehaviors()
 *****************************************************************************************************/
DLL_EXPORT void STD CargarBehaviors (BehaviorsParams *Parametros)
{
	sfStartBehavior(sfConstantVelocity,
					"Velocidad Constante",
					(*Parametros).ConstantVelocityParams.TimeOut,		//Timeout
					(*Parametros).ConstantVelocityParams.Prioridad,		//Prioridad
					(*Parametros).ConstantVelocityParams.Suspendido,	//Suspension
					(*Parametros).ConstantVelocityParams.Velocidad);	//Velocidad de traslacin

	sfStartBehavior(sfKeepOff,
					"Mantener Distancias",
					(*Parametros).KeepOffParams.TimeOut,		//Timeout
					(*Parametros).KeepOffParams.Prioridad,		//Prioridad
					(*Parametros).KeepOffParams.Suspendido,		//Suspension
					(*Parametros).KeepOffParams.VelocidadCauta,	//Velocidad de precaucion
					(*Parametros).KeepOffParams.Sensibilidad);	//Sensibilidad a los obstculos (de 0.2 a 2.0)

	sfStartBehavior(sfAvoidCollision,
					"Evita Colision",
					(*Parametros).AvoidCollisionParams.TimeOut,		//Timeout
					(*Parametros).AvoidCollisionParams.Prioridad,	//Prioriudad
					(*Parametros).AvoidCollisionParams.Suspendido,	//suspension
					(*Parametros).AvoidCollisionParams.SensibilidadFrontal, //Sensibilidad a obstculos frontales (de 0.5 a 3.0)
					(*Parametros).AvoidCollisionParams.SensibilidadLateral,	//Sensibilidad a obstculos laterales (de 0.5 a 3.0)
					(*Parametros).AvoidCollisionParams.GananciaGiro,	 	//Ganancia del giro (de 4.0 a 10.0)
					(*Parametros).AvoidCollisionParams.RadioSeguridad);		//Define la burbuja de seguridad del 
																					// robot en milmetros

	sfStartBehavior(sfStop,
					"Detener Robot",
					(*Parametros).StopParams.TimeOut,	//Timeout
					(*Parametros).StopParams.Prioridad,	//Prioriudad
					(*Parametros).StopParams.Suspendido);	//suspension
}

/**
 ******************************************************************
 * Descarga todos los comportamientos predefinidos de memoria
 * @see CargarBehaviors()
 *****************************************************************/
DLL_EXPORT void STD DescargarBehaviors ()
{
	sfRemoveTask ("Velocidad Constante");
	sfRemoveTask ("Mantener Distancias");
	sfRemoveTask ("Evita Colision");
	sfRemoveTask ("Detener Robot");
}

/*
DLL_EXPORT void STD	Behav_ConstVelocity (char *NombreInstancia,long TimeOut, 
										long Prioridad,long Suspension)
{
	ComportamientoVel=sfStartBehavior(sfConstantVelocity,
					NombreInstancia,
					(int)TimeOut,
					(int)Prioridad,
					(int)Suspension,
					100.0);
	sfLoadEvalFile("c:\\sfTrackingObject.dll");
	sfLoadEvalFile("c:\\sfTracking.act");
	i=sfTaskSuspended (NombreInstancia);
}
*/

/**
 *************************************************************************
 * Carga una actividad externa
 * @param NombreFichero Path completo del fichero que contiene la actividad
 * @see Behav_Kill()
 * @see Behav_Pause()
 * @see Behav_Resume()
 ************************************************************************/
DLL_EXPORT void STD Behav_Loaded (char *NombreFichero, char *NombreInstancia)
{
	sfLoadEvalFile (NombreFichero);
}

/**
 *****************************************************************************
 * Reactiva una actividad cargada en memoria que esta suspendida
 * @param NombreInstancia Nombre con que se conoce al comportamiento
 * @see Behav_Loaded()
 * @see Behav_Kill()
 * @see Behav_Pause()
 ****************************************************************************/
DLL_EXPORT void STD Behav_Resume (char *NombreInstancia)
{
	sfResumeTask (NombreInstancia);
}

/**
 *****************************************************************************
 * Elimina una actividad cargada en memoria
 * @param NombreInstancia Nombre con que se conoce al comportamiento
 * @see Behav_Loaded()
 * @see Behav_Pause()
 * @see Behav_Resume()
 ****************************************************************************/
DLL_EXPORT void STD Behav_Kill (char *NombreInstancia)
{
	sfRemoveTask (NombreInstancia);
}

/**
 *****************************************************************************
 * Suspende uno de las actividades cargadas en memoria
 * @param NombreInstancia Nombre con que se conoce al comportamiento
 * @see Behav_Loaded()
 * @see Behav_Kill()
 * @see Behav_Resume()
 ****************************************************************************/
DLL_EXPORT void STD Behav_Pause (char *NombreInstancia)
{
	sfInterruptTask (NombreInstancia);
}

/*DLL_EXPORT void STD splinetemp ()
{
//	sfLoadEvalFile("h:\\sfDoTrajectory.dll");

	sfDTAddGlobalPoint(0,0);

	sfDTAddGlobalPoint(0,0);
	sfDTAddGlobalPoint(1000,0);
	sfDTAddGlobalPoint(1500,0);
	sfDTAddGlobalPoint(2000,-500);
	sfDTAddGlobalPoint(2500,500);
	sfDTAddGlobalPoint(3000,0);
	sfDTAddGlobalPoint(3000,0);
	
	sfDTDrawTrajectory();
	sfDTStartTrajectory();
}
*/
/**
 ************************************************************************
 * Lanza el Thread encargado de seguir una Spline
 * @param x Coordenada x del punto de la Spline a alcanzar
 * @param y Coordenada y del punto de la Spline a alcanzar
 * @param vel Velocidad a la que hay que alcanzarlo
 * @param radio Distancia a la que se da por alcanzado el punto
 * @see ThreadSeguirSpline()
 * @see ThreadState()
 ************************************************************************/
 DLL_EXPORT void STD SeguirSpline (long x,long y,long th,double vel,double radio)
{
	ThreadFinished=false;
	Valores.x=y;
	Valores.y=-x;
	Valores.th=th;
	Valores.Velocidad=vel;
	Valores.RadioExito=radio;
	_beginthread (ThreadSeguirSpline,0,(void*)&Valores);
}

/**
 *************************************************************************
 * Thread que se encarga de seguir una Spline
 * @param Datos Puntero a estructura que contiene los datos necesarios 
 para seguir la Spline
 * @see SeguirSpline()
 * @see ThreadState()
 ************************************************************************/
void ThreadSeguirSpline (void *Datos)
{
	static sfprocess *p1=NULL;
	point *goal;
	bool Succes=false;
	PassToThreadType* Val = (PassToThreadType*)Datos;

	goal=sfCreateGlobalPoint ((float)(*Val).x,(float)(*Val).y,0);
	sfSetGlobalCoords(goal);        
	sfAddPointCheck(goal);
	p1=sfStartBehavior (sfAttendAtPos,
						"sfGotoPos",
						0,						//Timeout 
						5,						//Prioridad
						0,						//Suspensin
						(double)(*Val).Velocidad,		//Velocidad de traslacin
						goal,					//Punto de destino
						(double)(*Val).RadioExito);		//Radio de exito
   while (sfGetProcessState(p1)!=sfFAILURE && Succes==false) 
   {
		sfPause(1);	//Las cosas van mejor con una pausa aqui ;)
		switch(sfGetProcessState(p1)) 
		{
			case sfSUCCESS:
				if (sfFinished(p1)) 
				{
					p1=NULL;
					sfRemPoint(goal);
					Succes=true;
					break;
				}
            case sfTIMEOUT:
                p1=NULL;
                sfRemPoint(goal);
                break;
		}
	}
   ThreadFinished=true;
   _endthread();
}

/**
 ***********************************************************************
 * Nos dice si el Thread a llegado a su fin o no
 * @see SeguirSpline()
 * @see ThreadSeguirSpline()
 * @return 1 si ha acabado el Thread y 0 si no ha acabado
 ***********************************************************************/
DLL_EXPORT int STD ThreadState (void)
{
	if (ThreadFinished==true) {
		return 1;
	}
	return 0;
}
